@@ -5,6 +5,8 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module CmdLine (
 	dispatch,
 	usage,
@@ -15,7 +17,9 @@ import qualified Control.Exception as E
 import qualified Data.Map as M
 import Control.Exception (throw)
 import System.Console.GetOpt
+#ifndef __WINDOWS__
 import System.Posix.Signals
+#endif
 
 import Common.Annex
 import qualified Annex
@@ -24,6 +28,7 @@ import qualified Git
 import qualified Git.AutoCorrect
 import Annex.Content
 import Annex.Ssh
+import Annex.Environment
 import Command
 
 type Params = [String]
@@ -39,18 +44,21 @@ dispatch fuzzyok allargs allcmds commonoptions fields header getgitrepo = do
 		Right g -> do
 			state <- Annex.new g
 			(actions, state') <- Annex.run state $ do
+				checkEnvironment
 				checkfuzzy
-				forM_ fields $ \(f, v) -> Annex.setField f v
+				forM_ fields $ uncurry Annex.setField
 				sequence_ flags
+				whenM (annexDebug <$> Annex.getGitConfig) $
+					liftIO enableDebugOutput
 				prepCommand cmd params
 		 	tryRun state' cmd $ [startup] ++ actions ++ [shutdown $ cmdnocommit cmd]
-	where
-		err msg = msg ++ "\n\n" ++ usage header allcmds commonoptions
-		cmd = Prelude.head cmds
-		(fuzzy, cmds, name, args) = findCmd fuzzyok allargs allcmds err
-		(flags, params) = getOptCmd args cmd commonoptions err
-		checkfuzzy = when fuzzy $
-			inRepo $ Git.AutoCorrect.prepare name cmdname cmds
+  where
+	err msg = msg ++ "\n\n" ++ usage header allcmds
+	cmd = Prelude.head cmds
+	(fuzzy, cmds, name, args) = findCmd fuzzyok allargs allcmds err
+	(flags, params) = getOptCmd args cmd commonoptions
+	checkfuzzy = when fuzzy $
+		inRepo $ Git.AutoCorrect.prepare name cmdname cmds
 
 {- Parses command line params far enough to find the Command to run, and
  - returns the remaining params.
@@ -61,25 +69,28 @@ findCmd fuzzyok argv cmds err
 	| not (null exactcmds) = (False, exactcmds, fromJust name, args)
 	| fuzzyok && not (null inexactcmds) = (True, inexactcmds, fromJust name, args)
 	| otherwise = error $ err $ "unknown command " ++ fromJust name
-	where
-		(name, args) = findname argv []
-		findname [] c = (Nothing, reverse c)
-		findname (a:as) c
-			| "-" `isPrefixOf` a = findname as (a:c)
-			| otherwise = (Just a, reverse c ++ as)
-		exactcmds = filter (\c -> name == Just (cmdname c)) cmds
-		inexactcmds = case name of
-			Nothing -> []
-			Just n -> Git.AutoCorrect.fuzzymatches n cmdname cmds
+  where
+	(name, args) = findname argv []
+	findname [] c = (Nothing, reverse c)
+	findname (a:as) c
+		| "-" `isPrefixOf` a = findname as (a:c)
+		| otherwise = (Just a, reverse c ++ as)
+	exactcmds = filter (\c -> name == Just (cmdname c)) cmds
+	inexactcmds = case name of
+		Nothing -> []
+		Just n -> Git.AutoCorrect.fuzzymatches n cmdname cmds
 
 {- Parses command line options, and returns actions to run to configure flags
  - and the remaining parameters for the command. -}
-getOptCmd :: Params -> Command -> [Option] -> (String -> String) -> (Flags, Params)
-getOptCmd argv cmd commonoptions err = check $
+getOptCmd :: Params -> Command -> [Option] -> (Flags, Params)
+getOptCmd argv cmd commonoptions = check $
 	getOpt Permute (commonoptions ++ cmdoptions cmd) argv
-	where
-		check (flags, rest, []) = (flags, rest)
-		check (_, _, errs) = error $ err $ concat errs
+  where
+	check (flags, rest, []) = (flags, rest)
+	check (_, _, errs) = error $ unlines
+		[ concat errs
+		, commandUsage cmd
+		]
 
 {- Runs a list of Annex actions. Catches IO errors and continues
  - (but explicitly thrown errors terminate the whole command).
@@ -93,23 +104,25 @@ tryRun' errnum _ cmd []
 tryRun' errnum state cmd (a:as) = do
 	r <- run
 	handle $! r
-	where
-		run = tryIO $ Annex.run state $ do
-			Annex.Queue.flushWhenFull
-			a
-		handle (Left err) = showerr err >> cont False state
-		handle (Right (success, state')) = cont success state'
-		cont success s = do
-			let errnum' = if success then errnum else errnum + 1
-			(tryRun' $! errnum') s cmd as
-		showerr err = Annex.eval state $ do
-			showErr err
-			showEndFail
+  where
+	run = tryIO $ Annex.run state $ do
+		Annex.Queue.flushWhenFull
+		a
+	handle (Left err) = showerr err >> cont False state
+	handle (Right (success, state')) = cont success state'
+	cont success s = do
+		let errnum' = if success then errnum else errnum + 1
+		(tryRun' $! errnum') s cmd as
+	showerr err = Annex.eval state $ do
+		showErr err
+		showEndFail
 
 {- Actions to perform each time ran. -}
 startup :: Annex Bool
 startup = liftIO $ do
+#ifndef __WINDOWS__
 	void $ installHandler sigINT Default Nothing
+#endif
 	return True
 
 {- Cleanup actions. -}
@@ -117,5 +130,6 @@ shutdown :: Bool -> Annex Bool
 shutdown nocommit = do
 	saveState nocommit
 	sequence_ =<< M.elems <$> Annex.getState Annex.cleanup
+	liftIO reapZombies -- zombies from long-running git processes
 	sshCleanup -- ssh connection caching
 	return True
