@@ -15,21 +15,22 @@ import Annex.Wanted
 
 def :: [Command]
 def = [withOptions Command.Move.options $ command "copy" paramPaths seek
-	"copy content of files to/from another repository"]
+	SectionCommon "copy content of files to/from another repository"]
 
 seek :: [CommandSeek]
-seek = [withField Command.Move.toOption Remote.byName $ \to ->
-		withField Command.Move.fromOption Remote.byName $ \from ->
+seek = [withField Command.Move.toOption Remote.byNameWithUUID $ \to ->
+		withField Command.Move.fromOption Remote.byNameWithUUID $ \from ->
 			withFilesInGit $ whenAnnexed $ start to from]
 
 {- A copy is just a move that does not delete the source file.
  - However, --auto mode avoids unnecessary copies, and avoids getting or
  - sending non-preferred content. -}
 start :: Maybe Remote -> Maybe Remote -> FilePath -> (Key, Backend) -> CommandStart
-start to from file (key, backend) = autoCopies file key (<) $
-	stopUnless shouldCopy $ 
-		Command.Move.start to from False file (key, backend)
-	where
-		shouldCopy = case to of
-			Nothing -> checkAuto $ wantGet (Just file)
-			Just r -> checkAuto $ wantSend (Just file) (Remote.uuid r)
+start to from file (key, backend) = stopUnless shouldCopy $ 
+	Command.Move.start to from False file (key, backend)
+  where
+	shouldCopy = checkAuto (check <||> numCopiesCheck file key (<))
+	check = case to of
+		Nothing -> wantGet False (Just file)
+		Just r -> wantSend False (Just file) (Remote.uuid r)
+		
