@@ -9,18 +9,10 @@
 
 module GitAnnex where
 
-import System.Console.GetOpt
-
-import Common.Annex
-import qualified Git.Config
 import qualified Git.CurrentRepo
 import CmdLine
 import Command
-import Types.TrustLevel
-import qualified Annex
-import qualified Remote
-import qualified Limit
-import qualified Option
+import GitAnnex.Options
 
 import qualified Command.Add
 import qualified Command.Unannex
@@ -31,12 +23,16 @@ import qualified Command.Get
 import qualified Command.FromKey
 import qualified Command.DropKey
 import qualified Command.TransferKey
+#ifndef __WINDOWS__
+import qualified Command.TransferKeys
+#endif
 import qualified Command.ReKey
 import qualified Command.Reinject
 import qualified Command.Fix
 import qualified Command.Init
 import qualified Command.Describe
 import qualified Command.InitRemote
+import qualified Command.EnableRemote
 import qualified Command.Fsck
 import qualified Command.Unused
 import qualified Command.DropUnused
@@ -56,22 +52,33 @@ import qualified Command.Untrust
 import qualified Command.Semitrust
 import qualified Command.Dead
 import qualified Command.Group
+import qualified Command.Content
 import qualified Command.Ungroup
 import qualified Command.Vicfg
 import qualified Command.Sync
 import qualified Command.AddUrl
+import qualified Command.RmUrl
 import qualified Command.Import
 import qualified Command.Map
+import qualified Command.Direct
+import qualified Command.Indirect
 import qualified Command.Upgrade
 import qualified Command.Version
+import qualified Command.Help
 #ifdef WITH_ASSISTANT
 import qualified Command.Watch
 import qualified Command.Assistant
 #ifdef WITH_WEBAPP
 import qualified Command.WebApp
 #endif
+#ifdef WITH_XMPP
+import qualified Command.XMPPGit
+#endif
+#endif
+#ifdef WITH_TESTSUITE
+import qualified Command.Test
+import qualified Command.FuzzTest
 #endif
-import qualified Command.Help
 
 cmds :: [Command]
 cmds = concat
@@ -84,10 +91,12 @@ cmds = concat
 	, Command.Lock.def
 	, Command.Sync.def
 	, Command.AddUrl.def
+	, Command.RmUrl.def
 	, Command.Import.def
 	, Command.Init.def
 	, Command.Describe.def
 	, Command.InitRemote.def
+	, Command.EnableRemote.def
 	, Command.Reinject.def
 	, Command.Unannex.def
 	, Command.Uninit.def
@@ -97,11 +106,15 @@ cmds = concat
 	, Command.Semitrust.def
 	, Command.Dead.def
 	, Command.Group.def
+	, Command.Content.def
 	, Command.Ungroup.def
 	, Command.Vicfg.def
 	, Command.FromKey.def
 	, Command.DropKey.def
 	, Command.TransferKey.def
+#ifndef __WINDOWS__
+	, Command.TransferKeys.def
+#endif
 	, Command.ReKey.def
 	, Command.Fix.def
 	, Command.Fsck.def
@@ -115,58 +128,29 @@ cmds = concat
 	, Command.Status.def
 	, Command.Migrate.def
 	, Command.Map.def
+	, Command.Direct.def
+	, Command.Indirect.def
 	, Command.Upgrade.def
 	, Command.Version.def
+	, Command.Help.def
 #ifdef WITH_ASSISTANT
 	, Command.Watch.def
 	, Command.Assistant.def
 #ifdef WITH_WEBAPP
 	, Command.WebApp.def
 #endif
+#ifdef WITH_XMPP
+	, Command.XMPPGit.def
+#endif
+#endif
+#ifdef WITH_TESTSUITE
+	, Command.Test.def
+	, Command.FuzzTest.def
 #endif
-	, Command.Help.def
 	]
 
-options :: [Option]
-options = Option.common ++
-	[ Option ['N'] ["numcopies"] (ReqArg setnumcopies paramNumber)
-		"override default number of copies"
-	, Option [] ["trust"] (ReqArg (Remote.forceTrust Trusted) paramRemote)
-		"override trust setting"
-	, Option [] ["semitrust"] (ReqArg (Remote.forceTrust SemiTrusted) paramRemote)
-		"override trust setting back to default"
-	, Option [] ["untrust"] (ReqArg (Remote.forceTrust UnTrusted) paramRemote)
-		"override trust setting to untrusted"
-	, Option ['c'] ["config"] (ReqArg setgitconfig "NAME=VALUE")
-		"override git configuration setting"
-	, Option ['x'] ["exclude"] (ReqArg Limit.addExclude paramGlob)
-		"skip files matching the glob pattern"
-	, Option ['I'] ["include"] (ReqArg Limit.addInclude paramGlob)
-		"don't skip files matching the glob pattern"
-	, Option ['i'] ["in"] (ReqArg Limit.addIn paramRemote)
-		"skip files not present in a remote"
-	, Option ['C'] ["copies"] (ReqArg Limit.addCopies paramNumber)
-		"skip files with fewer copies"
-	, Option ['B'] ["inbackend"] (ReqArg Limit.addInBackend paramName)
-		"skip files not using a key-value backend"
-	, Option [] ["inallgroup"] (ReqArg Limit.addInAllGroup paramGroup)
-		"skip files not present in all remotes in a group"
-	, Option [] ["largerthan"] (ReqArg Limit.addLargerThan paramSize)
-		"skip files larger than a size"
-	, Option [] ["smallerthan"] (ReqArg Limit.addSmallerThan paramSize)
-		"skip files smaller than a size"
-	, Option ['T'] ["time-limit"] (ReqArg Limit.addTimeLimit paramTime)
-		"stop after the specified amount of time"
-	] ++ Option.matcher
-	where
-		setnumcopies v = Annex.changeState $ \s -> s { Annex.forcenumcopies = readish v }
-		setgitconfig :: String -> Annex ()
-		setgitconfig v = do
-			newg <- inRepo $ Git.Config.store v
-			Annex.changeState $ \s -> s { Annex.repo = newg }
-
 header :: String
-header = "Usage: git-annex command [option ..]"
+header = "git-annex command [option ...]"
 
 run :: [String] -> IO ()
 run args = dispatch True args cmds options [] header Git.CurrentRepo.get
