@@ -19,15 +19,16 @@ module Remote (
 
 	remoteTypes,
 	remoteList,
-	enabledRemoteList,
 	specialRemote,
 	remoteMap,
 	uuidDescriptions,
 	byName,
+	byNameWithUUID,
 	byCost,
 	prettyPrintUUIDs,
 	prettyListUUIDs,
-	repoFromUUID,
+	prettyUUID,
+	remoteFromUUID,
 	remotesWithUUID,
 	remotesWithoutUUID,
 	keyLocations,
@@ -44,6 +45,7 @@ import qualified Data.Map as M
 import Text.JSON
 import Text.JSON.Generic
 import Data.Tuple
+import Data.Ord
 
 import Common.Annex
 import Types.Remote
@@ -51,9 +53,8 @@ import qualified Annex
 import Annex.UUID
 import Logs.UUID
 import Logs.Trust
-import Logs.Location
+import Logs.Location hiding (logStatus)
 import Remote.List
-import qualified Git
 
 {- Map from UUIDs of Remotes to a calculated value. -}
 remoteMap :: (Remote -> a) -> Annex (M.Map UUID a)
@@ -73,17 +74,28 @@ addName desc n
 	| otherwise = n ++ " (" ++ desc ++ ")"
 
 {- When a name is specified, looks up the remote matching that name.
- - (Or it can be a UUID.) Only finds currently configured git remotes. -}
+ - (Or it can be a UUID.) -}
 byName :: Maybe String -> Annex (Maybe Remote)
 byName Nothing = return Nothing
 byName (Just n) = either error Just <$> byName' n
+
+{- Like byName, but the remote must have a configured UUID. -}
+byNameWithUUID :: Maybe String -> Annex (Maybe Remote)
+byNameWithUUID n = do
+	v <- byName n
+	return $ checkuuid <$> v
+  where
+	checkuuid r
+		| uuid r == NoUUID = error $ "cannot determine uuid for " ++ name r
+		| otherwise = r
+
 byName' :: String -> Annex (Either String Remote)
 byName' "" = return $ Left "no remote specified"
 byName' n = handle . filter matching <$> remoteList
-	where
-		handle [] = Left $ "there is no available git remote named \"" ++ n ++ "\""
-		handle match = Right $ Prelude.head match
-		matching r = n == name r || toUUID n == uuid r
+  where
+	handle [] = Left $ "there is no available git remote named \"" ++ n ++ "\""
+	handle (match:_) = Right match
+	matching r = n == name r || toUUID n == uuid r
 
 {- Looks up a remote by name (or by UUID, or even by description),
  - and returns its UUID. Finds even remotes that are not configured in
@@ -93,17 +105,17 @@ nameToUUID "." = getUUID -- special case for current repo
 nameToUUID "here" = getUUID
 nameToUUID "" = error "no remote specified"
 nameToUUID n = byName' n >>= go
-	where
-		go (Right r) = return $ uuid r
-		go (Left e) = fromMaybe (error e) <$> bydescription
-		bydescription = do
-			m <- uuidMap
-			case M.lookup n $ transform swap m of
-				Just u -> return $ Just u
-				Nothing -> return $ byuuid m
-		byuuid m = M.lookup (toUUID n) $ transform double m
-		transform a = M.fromList . map a . M.toList
-		double (a, _) = (a, a)
+  where
+	go (Right r) = return $ uuid r
+	go (Left e) = fromMaybe (error e) <$> bydescription
+	bydescription = do
+		m <- uuidMap
+		case M.lookup n $ transform swap m of
+			Just u -> return $ Just u
+			Nothing -> return $ byuuid m
+	byuuid m = M.lookup (toUUID n) $ transform double m
+	transform a = M.fromList . map a . M.toList
+	double (a, _) = (a, a)
 
 {- Pretty-prints a list of UUIDs of remotes, for human display.
  -
@@ -115,47 +127,48 @@ prettyPrintUUIDs desc uuids = do
 	m <- uuidDescriptions
 	maybeShowJSON [(desc, map (jsonify m hereu) uuids)]
 	return $ unwords $ map (\u -> "\t" ++ prettify m hereu u ++ "\n") uuids
-	where
-		finddescription m u = M.findWithDefault "" u m
-		prettify m hereu u
-			| not (null d) = fromUUID u ++ " -- " ++ d
-			| otherwise = fromUUID u
-			where
-				ishere = hereu == u
-				n = finddescription m u
-				d
-					| null n && ishere = "here"
-					| ishere = addName n "here"
-					| otherwise = n
-		jsonify m hereu u = toJSObject
-			[ ("uuid", toJSON $ fromUUID u)
-			, ("description", toJSON $ finddescription m u)
-			, ("here", toJSON $ hereu == u)
-			]
+  where
+	finddescription m u = M.findWithDefault "" u m
+	prettify m hereu u
+		| not (null d) = fromUUID u ++ " -- " ++ d
+		| otherwise = fromUUID u
+	  where
+		ishere = hereu == u
+		n = finddescription m u
+		d
+			| null n && ishere = "here"
+			| ishere = addName n "here"
+			| otherwise = n
+	jsonify m hereu u = toJSObject
+		[ ("uuid", toJSON $ fromUUID u)
+		, ("description", toJSON $ finddescription m u)
+		, ("here", toJSON $ hereu == u)
+		]
 
 {- List of remote names and/or descriptions, for human display.  -}
 prettyListUUIDs :: [UUID] -> Annex [String]
 prettyListUUIDs uuids = do
 	hereu <- getUUID
 	m <- uuidDescriptions
-	return $ map (\u -> prettify m hereu u) uuids
-	where
-		finddescription m u = M.findWithDefault "" u m
-		prettify m hereu u
-			| u == hereu = addName n "here"
-			| otherwise = n
-			where
-				n = finddescription m u
+	return $ map (prettify m hereu) uuids
+  where
+	finddescription m u = M.findWithDefault "" u m
+	prettify m hereu u
+		| u == hereu = addName n "here"
+		| otherwise = n
+	  where
+		n = finddescription m u
 
-{- Gets the git repo associated with a UUID.
+{- Nice display of a remote's name and/or description. -}
+prettyUUID :: UUID -> Annex String
+prettyUUID u = concat <$> prettyListUUIDs [u]
+
+{- Gets the remote associated with a UUID.
  - There's no associated remote when this is the UUID of the local repo. -}
-repoFromUUID :: UUID -> Annex (Git.Repo, Maybe Remote)
-repoFromUUID u = ifM ((==) u <$> getUUID)
-	( (,) <$> gitRepo <*> pure Nothing
-	, do
-		remote <- fromMaybe (error "Unknown UUID") . M.lookup u
-			<$> remoteMap id
-		return (repo remote, Just remote)
+remoteFromUUID :: UUID -> Annex (Maybe Remote)
+remoteFromUUID u = ifM ((==) u <$> getUUID)
+	( return Nothing
+	, Just . fromMaybe (error "Unknown UUID") . M.lookup u <$> remoteMap id
 	)
 
 {- Filters a list of remotes to ones that have the listed uuids. -}
@@ -169,7 +182,7 @@ remotesWithoutUUID rs us = filter (\r -> uuid r `notElem` us) rs
 {- List of repository UUIDs that the location log indicates may have a key.
  - Dead repositories are excluded. -}
 keyLocations :: Key -> Annex [UUID]
-keyLocations key = snd <$> (trustPartition DeadTrusted =<< loggedLocations key)
+keyLocations key = trustExclude DeadTrusted =<< loggedLocations key
 
 {- Cost ordered lists of remotes that the location log indicates
  - may have a key.
@@ -197,14 +210,15 @@ keyPossibilities' key trusted = do
 	let validtrusteduuids = validuuids `intersect` trusted
 
 	-- remotes that match uuids that have the key
-	allremotes <- enabledRemoteList
+	allremotes <- filter (not . remoteAnnexIgnore . gitconfig)
+		<$> remoteList
 	let validremotes = remotesWithUUID allremotes validuuids
 
-	return (sort validremotes, validtrusteduuids)
+	return (sortBy (comparing cost) validremotes, validtrusteduuids)
 
 {- Displays known locations of a key. -}
-showLocations :: Key -> [UUID] -> Annex ()
-showLocations key exclude = do
+showLocations :: Key -> [UUID] -> String -> Annex ()
+showLocations key exclude nolocmsg = do
 	u <- getUUID
 	uuids <- keyLocations key
 	untrusteduuids <- trustGet UnTrusted
@@ -213,18 +227,18 @@ showLocations key exclude = do
 	ppuuidswanted <- Remote.prettyPrintUUIDs "wanted" uuidswanted
 	ppuuidsskipped <- Remote.prettyPrintUUIDs "skipped" uuidsskipped
 	showLongNote $ message ppuuidswanted ppuuidsskipped
-	where
-		filteruuids l x = filter (`notElem` x) l
-		message [] [] = "No other repository is known to contain the file."
-		message rs [] = "Try making some of these repositories available:\n" ++ rs
-		message [] us = "Also these untrusted repositories may contain the file:\n" ++ us
-		message rs us = message rs [] ++ message [] us
+  where
+	filteruuids l x = filter (`notElem` x) l
+	message [] [] = nolocmsg
+	message rs [] = "Try making some of these repositories available:\n" ++ rs
+	message [] us = "Also these untrusted repositories may contain the file:\n" ++ us
+	message rs us = message rs [] ++ message [] us
 
 showTriedRemotes :: [Remote] -> Annex ()
 showTriedRemotes [] = noop
 showTriedRemotes remotes =
 	showLongNote $ "Unable to access these remotes: " ++
-		join ", " (map name remotes)
+		intercalate ", " (map name remotes)
 
 forceTrust :: TrustLevel -> String -> Annex ()
 forceTrust level remotename = do
@@ -241,7 +255,7 @@ logStatus remote key = logChange key (uuid remote)
 
 {- Orders remotes by cost, with ones with the lowest cost grouped together. -}
 byCost :: [Remote] -> [[Remote]]
-byCost = map snd . sort . M.toList . costmap
-	where
-		costmap = M.fromListWith (++) . map costpair
-		costpair r = (cost r, [r])
+byCost = map snd . sortBy (comparing fst) . M.toList . costmap
+  where
+	costmap = M.fromListWith (++) . map costpair
+	costpair r = (cost r, [r])
