@@ -15,8 +15,12 @@ import Data.Ord
 import qualified Git
 import Types.Key
 import Types.UUID
+import Types.GitConfig
+import Config.Cost
+import Utility.Metered
 
-type RemoteConfig = M.Map String String
+type RemoteConfigKey = String
+type RemoteConfig = M.Map RemoteConfigKey String
 
 {- There are different types of remotes. -}
 data RemoteTypeA a = RemoteType {
@@ -25,7 +29,7 @@ data RemoteTypeA a = RemoteType {
 	-- enumerates remotes of this type
 	enumerate :: a [Git.Repo],
 	-- generates a remote of this type
-	generate :: Git.Repo -> UUID -> Maybe RemoteConfig -> a (RemoteA a),
+	generate :: Git.Repo -> UUID -> RemoteConfig -> RemoteGitConfig -> a (RemoteA a),
 	-- initializes or changes a remote
 	setup :: UUID -> RemoteConfig -> a RemoteConfig
 }
@@ -33,6 +37,9 @@ data RemoteTypeA a = RemoteType {
 instance Eq (RemoteTypeA a) where
 	x == y = typename x == typename y
 
+{- A filename associated with a Key, for display to user. -}
+type AssociatedFile = Maybe FilePath
+
 {- An individual remote. -}
 data RemoteA a = Remote {
 	-- each Remote has a unique uuid
@@ -40,11 +47,13 @@ data RemoteA a = Remote {
 	-- each Remote has a human visible name
 	name :: String,
 	-- Remotes have a use cost; higher is more expensive
-	cost :: Int,
+	cost :: Cost,
 	-- Transfers a key to the remote.
-	storeKey :: Key -> a Bool,
-	-- retrieves a key's contents to a file
-	retrieveKeyFile :: Key -> FilePath -> a Bool,
+	storeKey :: Key -> AssociatedFile -> MeterUpdate -> a Bool,
+	-- Retrieves a key's contents to a file.
+	-- (The MeterUpdate does not need to be used if it retrieves
+	-- directly to the file, and not to an intermediate file.)
+	retrieveKeyFile :: Key -> AssociatedFile -> FilePath -> MeterUpdate -> a Bool,
 	-- retrieves a key's contents to a tmp file, if it can be done cheaply
 	retrieveKeyFileCheap :: Key -> FilePath -> a Bool,
 	-- removes a key's contents
@@ -55,10 +64,20 @@ data RemoteA a = Remote {
 	-- Some remotes can check hasKey without an expensive network
 	-- operation.
 	hasKeyCheap :: Bool,
-	-- a Remote can have a persistent configuration store
-	config :: Maybe RemoteConfig,
-	-- git configuration for the remote
+	-- Some remotes can provide additional details for whereis.
+	whereisKey :: Maybe (Key -> a [String]),
+	-- a Remote has a persistent configuration store
+	config :: RemoteConfig,
+	-- git repo for the Remote
 	repo :: Git.Repo,
+	-- a Remote's configuration from git
+	gitconfig :: RemoteGitConfig,
+	-- a Remote can be assocated with a specific local filesystem path
+	localpath :: Maybe FilePath,
+	-- a Remote can be known to be readonly
+	readonly :: Bool,
+	-- a Remote can be globally available. (Ie, "in the cloud".)
+	globallyAvailable :: Bool,
 	-- the type of the remote
 	remotetype :: RemoteTypeA a
 }
@@ -70,6 +89,5 @@ instance Show (RemoteA a) where
 instance Eq (RemoteA a) where
 	x == y = uuid x == uuid y
 
--- order remotes by cost
 instance Ord (RemoteA a) where
-	compare = comparing cost
+	compare = comparing uuid
