@@ -5,23 +5,31 @@
  - Licensed under the GNU GPL version 3 or higher.
  -}
 
+{-# LANGUAGE CPP #-}
+
 module Utility.Gpg where
 
-import qualified Data.ByteString.Lazy.Char8 as L
 import System.Posix.Types
 import Control.Applicative
 import Control.Concurrent
-import Control.Exception (finally, bracket)
-import System.Exit
-import System.Posix.Env (setEnv, unsetEnv, getEnv)	
+import Control.Exception (bracket)
+import System.Path
 
 import Common
+import Utility.Env
+import qualified Build.SysConfig as SysConfig
 
 newtype KeyIds = KeyIds [String]
-        deriving (Ord, Eq)
+	deriving (Ord, Eq)
+
+{- If a specific gpg command was found at configure time, use it.
+ - Otherwise, try to run gpg. -}
+gpgcmd :: FilePath
+gpgcmd = fromMaybe "gpg" SysConfig.gpg
 
 stdParams :: [CommandParam] -> IO [String]
 stdParams params = do
+#ifndef __WINDOWS__
 	-- Enable batch mode if GPG_AGENT_INFO is set, to avoid extraneous
 	-- gpg output about password prompts. GPG_BATCH is set by the test
 	-- suite for a similar reason.
@@ -29,83 +37,115 @@ stdParams params = do
 	b <- getEnv "GPG_BATCH"
 	let batch = if isNothing e && isNothing b
 		then []
-		else ["--batch", "--no-tty"]
+		else ["--batch", "--no-tty", "--use-agent"]
 	return $ batch ++ defaults ++ toCommand params
-	where
-		-- be quiet, even about checking the trustdb
-		defaults = ["--quiet", "--trust-model", "always"]
+#else
+	return $ defaults ++ toCommand params
+#endif
+  where
+	-- be quiet, even about checking the trustdb
+	defaults = ["--quiet", "--trust-model", "always"]
 
 {- Runs gpg with some params and returns its stdout, strictly. -}
 readStrict :: [CommandParam] -> IO String
 readStrict params = do
 	params' <- stdParams params
-	pOpen ReadFromPipe "gpg" params' hGetContentsStrict
+	withHandle StdoutHandle createProcessSuccess (proc gpgcmd params') $ \h -> do
+		hSetBinaryMode h True
+		hGetContentsStrict h
 
 {- Runs gpg, piping an input value to it, and returning its stdout,
  - strictly. -}
 pipeStrict :: [CommandParam] -> String -> IO String
 pipeStrict params input = do
 	params' <- stdParams params
-	(pid, fromh, toh) <- hPipeBoth "gpg" params'
-	_ <- forkIO $ finally (hPutStr toh input) (hClose toh)
-	output <- hGetContentsStrict fromh
-	forceSuccess pid
-	return output
+	withBothHandles createProcessSuccess (proc gpgcmd params') $ \(to, from) -> do
+		hSetBinaryMode to True
+		hSetBinaryMode from True
+		hPutStr to input
+		hClose to
+		hGetContentsStrict from
 
-{- Runs gpg with some parameters, first feeding it a passphrase via
- - --passphrase-fd, then feeding it an input, and passing a handle
- - to its output to an action.
+{- Runs gpg with some parameters. First sends it a passphrase via
+ - --passphrase-fd. Then runs a feeder action that is passed a handle and
+ - should write to it all the data to input to gpg. Finally, runs
+ - a reader action that is passed a handle to gpg's output. 
+ -
+ - Runs gpg in batch mode; this is necessary to avoid gpg 2.x prompting for
+ - the passphrase.
  -
  - Note that to avoid deadlock with the cleanup stage,
- - the action must fully consume gpg's input before returning. -}
-passphraseHandle :: [CommandParam] -> String -> IO L.ByteString -> (Handle -> IO a) -> IO a
-passphraseHandle params passphrase a b = do
+ - the reader must fully consume gpg's input before returning. -}
+feedRead :: [CommandParam] -> String -> (Handle -> IO ()) -> (Handle -> IO a) -> IO a
+#ifndef __WINDOWS__
+feedRead params passphrase feeder reader = do
 	-- pipe the passphrase into gpg on a fd
 	(frompipe, topipe) <- createPipe
-	_ <- forkIO $ do
+	void $ forkIO $ do
 		toh <- fdToHandle topipe
 		hPutStrLn toh passphrase
 		hClose toh
 	let Fd pfd = frompipe
 	let passphrasefd = [Param "--passphrase-fd", Param $ show pfd]
 
-	params' <- stdParams $ passphrasefd ++ params
-	(pid, fromh, toh) <- hPipeBoth "gpg" params'
-	pid2 <- forkProcess $ do
-		L.hPut toh =<< a
-		hClose toh
-		exitSuccess
-	hClose toh
-	ret <- b fromh
-
-	-- cleanup
-	forceSuccess pid
-	_ <- getProcessStatus True False pid2
-	closeFd frompipe
-	return ret
+	params' <- stdParams $ [Param "--batch"] ++ passphrasefd ++ params
+	closeFd frompipe `after`
+		withBothHandles createProcessSuccess (proc gpgcmd params') go
+  where
+	go (to, from) = do
+		void $ forkIO $ do
+			feeder to
+			hClose to
+		reader from
+#else
+feedRead = error "gpg feedRead not implemented on Windows" -- TODO
+#endif
 
 {- Finds gpg public keys matching some string. (Could be an email address,
- - a key id, or a name. -}
+ - a key id, or a name; See the section 'HOW TO SPECIFY A USER ID' of
+ - GnuPG's manpage.) -}
 findPubKeys :: String -> IO KeyIds
 findPubKeys for = KeyIds . parse <$> readStrict params
-	where
-		params = [Params "--with-colons --list-public-keys", Param for]
-		parse = map keyIdField . filter pubKey . lines
-		pubKey = isPrefixOf "pub:"
-		keyIdField s = split ":" s !! 4
+  where
+	params = [Params "--with-colons --list-public-keys", Param for]
+	parse = catMaybes . map (keyIdField . split ":") . lines
+	keyIdField ("pub":_:_:_:f:_) = Just f
+	keyIdField _ = Nothing
 
 {- Creates a block of high-quality random data suitable to use as a cipher.
  - It is armored, to avoid newlines, since gpg only reads ciphers up to the
  - first newline. -}
-genRandom :: Int -> IO String
-genRandom size = readStrict
-	[ Params "--gen-random --armor"
+genRandom :: Bool -> Int -> IO String
+genRandom highQuality size = checksize <$> readStrict
+	[ Params params
 	, Param $ show randomquality
 	, Param $ show size
 	]
-	where
-		-- 1 is /dev/urandom; 2 is /dev/random
-		randomquality = 1 :: Int
+  where
+	params = "--gen-random --armor"
+
+	-- See http://www.gnupg.org/documentation/manuals/gcrypt/Quality-of-random-numbers.html
+	-- for the meaning of random quality levels.
+	-- The highest available is 2, which is the default for OpenPGP
+	-- key generation; Note that it uses the blocking PRNG /dev/random
+	-- on the Linux kernel, hence the running time may take a while.
+	randomquality :: Int
+	randomquality = if highQuality then 2 else 1
+
+ 	{- The size is the number of bytes of entropy desired; the data is
+	 - base64 encoded, so needs 8 bits to represent every 6 bytes of
+	 - entropy. -}
+	expectedlength = size * 8 `div` 6
+
+	checksize s = let len = length s in
+		if len >= expectedlength
+			then s
+			else shortread len
+
+	shortread got = error $ unwords
+		[ "Not enough bytes returned from gpg", params
+		, "(got", show got, "; expected", show expectedlength, ")"
+		]
 
 {- A test key. This is provided pre-generated since generating a new gpg
  - key is too much work (requires too much entropy) for a test suite to
@@ -176,34 +216,36 @@ keyBlock public ls = unlines
 	, unlines ls
 	, "-----END PGP "++t++" KEY BLOCK-----"
 	]
-	where
-		t
-			| public = "PUBLIC"
-			| otherwise = "PRIVATE"
+  where
+	t
+		| public = "PUBLIC"
+		| otherwise = "PRIVATE"
 
+#ifndef mingw32_HOST_OS
 {- Runs an action using gpg in a test harness, in which gpg does
  - not use ~/.gpg/, but a directory with the test key set up to be used. -}
 testHarness :: IO a -> IO a
 testHarness a = do
 	orig <- getEnv var
 	bracket setup (cleanup orig) (const a)
-	where
-		var = "GNUPGHOME"		
+  where
+	var = "GNUPGHOME"		
 
-		setup = do
-			base <- getTemporaryDirectory
-			dir <- mktmpdir $ base </> "gpgtmpXXXXXX"
-			setEnv var dir True
-			_ <- pipeStrict [Params "--import -q"] $ unlines
-				[testSecretKey, testKey]
-			return dir
+	setup = do
+		base <- getTemporaryDirectory
+		dir <- mktmpdir $ base </> "gpgtmpXXXXXX"
+		void $ setEnv var dir True
+		_ <- pipeStrict [Params "--import -q"] $ unlines
+			[testSecretKey, testKey]
+		return dir
 		
-		cleanup orig tmpdir = removeDirectoryRecursive tmpdir >> reset orig
-                reset (Just v) = setEnv var v True
-                reset _ = unsetEnv var
+	cleanup orig tmpdir = removeDirectoryRecursive tmpdir >> reset orig
+	reset (Just v) = setEnv var v True
+	reset _ = unsetEnv var
 
 {- Tests the test harness. -}
 testTestHarness :: IO Bool
 testTestHarness = do
 	keys <- testHarness $ findPubKeys testKeyId
 	return $ KeyIds [testKeyId] == keys
+#endif
